#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/icmp.h>

MODULE_DESCRIPTION("Simple netfilter module");
MODULE_AUTHOR("SO2");
MODULE_LICENSE("GPL");

#define LOG_LEVEL		KERN_ALERT

/*
 * Count ICMP echo packets and drop every 2nd packet.
 */

static unsigned int icmp_drop_hookfn(const struct nf_hook_ops *ops,
		struct sk_buff *skb,
		const struct net_device *in,
		const struct net_device *out,
		int (*okfn)(struct sk_buff *))
{
	static size_t count = 0;	/* ICMP packets counter */
	/* Get IP header. */
	struct iphdr *iph = ip_hdr(skb);

	if (iph->protocol == IPPROTO_ICMP) {
		/* Get ICMP header. */
		struct icmphdr *icmph = icmp_hdr(skb);
		/* If ICMP echo and in case of even counter, drop packet. */
		if (icmph->type == ICMP_ECHO) {
			count++;
			if (count % 2 == 0)
				return NF_DROP;
		}
	}

	/* By default, let the packet pass. */
	return NF_ACCEPT;
}

static struct nf_hook_ops icmp_nf_ops = {
	.owner       = THIS_MODULE,
	.hook        = icmp_drop_hookfn,
	.hooknum     = NF_INET_LOCAL_IN,
	.pf          = PF_INET,
	.priority    = NF_IP_PRI_FIRST
};

int __init hook_init(void)
{
	int err;

	/* register netfilter hook */
	err = nf_register_hook(&icmp_nf_ops);
	if (err)
		return err;

	return 0;
}

void __exit hook_exit(void)
{
	/* unregister hook */
	nf_unregister_hook(&icmp_nf_ops);
}

module_init(hook_init);
module_exit(hook_exit);
